<?php

// Setup the plugin.
add_action( 'plugins_loaded', 'wpex_replace_setup' );
function wpex_replace_setup() 
{
  // Load translation 
	load_plugin_textdomain( 'wpex-replace', false, 'wpex-replace/languages' );
}

//
// load JS and CSS
//

add_action( 'admin_enqueue_scripts', 'wpex_replace_load_js_and_css' );
function wpex_replace_load_js_and_css() {
	global $hook_suffix;

	if ( in_array( $hook_suffix, array( 
		'tools_page_wpex_replace_string_tool',
	) ) )
	{
		wp_enqueue_style('jquery-ui', WPEX_REPLACE_PLUGIN_URL.'/css/jquery/jquery-ui.css', false, '1.9.2', false);
		wp_enqueue_script("jquery");
		wp_enqueue_script("jquery-ui-core");
		wp_enqueue_script("jquery-ui-tabs");

		wp_register_style( 'wpex-replace.css', WPEX_REPLACE_PLUGIN_URL . 'css/wpex-replace.css' );
		wp_enqueue_style( 'wpex-replace.css' );
	
		wp_register_script( 'wpex-replace.js', WPEX_REPLACE_PLUGIN_URL . 'js/wpex-replace.js' );
		wp_enqueue_script( 'wpex-replace.js' );
		wp_localize_script( 'wpex-replace.js', 'wpxe_replace_i18n', array(
      'ajax_url' => home_url().'/wp-admin/admin-ajax.php',
			'invalid_url' => __("Invalid url '{url}'", 'wpex-replace'),
			'no_urls_replaced' => __('No urls replaced.', 'wpex-replace'),
		));
	}
}

//
// Admin Menu
//

add_action('admin_menu', 'wpex_replace_admin_menu');
function wpex_replace_admin_menu()
{
	add_submenu_page(
		'tools.php',
		__( 'Replace Url', 'wpex-replace' ),
		__( 'Replace Urls', 'wpex-replace' ),
		'create_users',
		'wpex_replace_string_tool',
    'wpex_replace_string_display_tool'
	);
}

//
// Show Tools -> Replace Urls
//

function wpex_replace_string_display_tool()
{
?>
  <div id="wpex-replace-string-tools" class="wrap">
		<div id="icon-tools" class="icon32"><br/></div>
		<h2><?php _e('Replace Urls', 'wpex-replace'); ?></h2>

		<div class="caution"><p><?php _e('<b>CAUTION:</b> The changes are written directly into the database. Be sure to make a backup before running the replacement!', 'wpex-replace'); ?></p></div>

		<p>
			<?php _e('Here you can replace a URL with another. This is sometimes necessary, for example when you change the domain name.', 'wpex-replace'); ?><br/>
			<?php _e('You can exclude a URL from the search by clicking the Exclude button. Or leave the replacement field blank. Only URLs with a new valid URL will be replaced.', 'wpex-replace'); ?>
		</p>
		
		<div id="wpex-replace-tabs">
			<ul>
				<li><a href="#wpex-replace-tab-general"><?php _e('General', 'wpex-replace'); ?></a></li>
				<li><a href="#wpex-replace-tab-excludes"><?php _e('Excluded Urls', 'wpex-replace'); ?></a></li>
			</ul>
			
			<?php /* --- General Tab --- */ ?>
			<div id="wpex-replace-tab-general">
	
				<?php /* --- Search URLs --- */ ?>
				<div class="tab-row">
					<span class="message-search-urls"><?php _e('If you want to find all urls in the complete database, press the following button.', 'wpex-replace'); ?></span>
					<input id="btn-search-urls" name="btn-search-urls" type="submit" class="button button-primary" value="<?php _e('Search database for urls', 'wpex-replace'); ?>" />
					<div class="list-search-urls"></div>
				</div>
				
				<?php /* --- Search String --- * / ?>
				<div class="tab-row">
					<form method="post" action="'.$_SERVER['REQUEST_URI'].'">
						<p><?php _e('xxx', 'wpex-replace'); ?></p>
						<input id="btn-search-urls" name="btn-search-urls" type="submit" class="button button-primary" value="<?php _e('Search database for urls', 'wpex-replace'); ?>" />
					</form>
				</div>
				<?php */ ?>
			</div>
			
			<?php /* --- Excluded Urls Tab --- */ ?>
			<div id="wpex-replace-tab-excludes">
				<!-- filled via ajax -->
			</div>
		</div>
	</div>
<?php		
}

//
// Ajax
//


// creating Ajax call for searching all urls in database
add_action( 'wp_ajax_wpex_replace_search_urls', 'wpex_replace_ajax_search_urls' );
function wpex_replace_ajax_search_urls()
{
	$htmlContent = '';
	$pos = 0;
	$r = new WPEXReplaceTextInTables();
	$urlList = $r->findURls();
	foreach ($urlList as $url=>$tables)
	{
		$htmlContent .= '
		<div id="'.$pos.'" class="row">
			<input type="text" id="text_'.$pos.'" name="text_'.$pos.'" value="'.$url.'" readonly="readonly"';
		
		$tIndex = 0;
		$htmlContent .= 'ids="';
		foreach ($tables as $table=>$ids)
		{
			if ($tIndex > 0)
				$htmlContent .= ';';
			$htmlContent .= "{$table}[{$ids[0]['name']}]=";
			foreach ($ids as $index=>$id)
			{
				if ($index > 0)
					$htmlContent .= ',';
				$htmlContent .= $id['value'];
			}
			$tIndex++;
		}
		$htmlContent .= ';" />
			<input type="text" id="newtext_'.$pos.'" name="newtext_'.$pos.'" value="" />
			<input type="button" class="button add" id="btntext_'.$pos.'" name="btntext_'.$pos.'" value="'.__('Exclude', 'wpex-replace').'" />
		</div>';
		
		$pos++;
	}
	$htmlContent .= '<div class="no-found'.(empty($htmlContent) ? '' : ' hidden').'">'.__('No Urls found.', 'wpex-replace').'</div>';
		
	$html = '
		<h3>'.__('Founded Urls', 'wpex-replace').'</h3>
		<div class="replacing-urls">'.$htmlContent.'</div>
		<div class="buttons">
			<input type="button" class="button button-primary" id="btnreplaceurls" name="btnreplaceurls" value="'.__('Replace', 'wpex-replace').'" />
		</div>';			
	
	die(json_encode(array(
		'html' => $html,
	)));
}

// creating Ajax call for get a list of excluded urls
add_action( 'wp_ajax_wpex_replace_get_excluded_search_urls', 'wpex_replace_ajax_get_excluded_search_urls' );
function wpex_replace_ajax_get_excluded_search_urls()
{
	$urls = unserialize(get_option('wpex_replace_exclude_search_urls'));
	if (empty($urls))
		$urls = array();

	$pos = 0;
	$htmlContent = '';
	foreach ($urls as $url)
	{
		$htmlContent .= '
		<div class="row" id="'.$pos.'">
			<input type="text" id="text_'.$pos.'" name="text_'.$pos.'" value="'.$url.'" readonly="readonly"/>
			<input type="button" class="button remove" id="btntext_'.$pos.'" name="btntext_'.$pos.'" value="'.__('Remove', 'wpex-replace').'" />
		</div>';
		$pos++;
	}
	$htmlContent .= '<div class="no-found'.(empty($htmlContent) ? '' : ' hidden').'">'.__('No excluded urls found.', 'wpex-replace').'</div>';

	$html = '<h3>'.__('Excluded Urls', 'wpex-replace').'</h3><div id="wpex-replace-excludeed-urls">'.$htmlContent.'</div>';
		
	die(json_encode(array(
		'html' => $html,
	)));
}

// creating Ajax call for exclude url from search
add_action( 'wp_ajax_wpex_replace_exclude_search_url', 'wpex_replace_ajax_exclude_search_urls' );
function wpex_replace_ajax_exclude_search_urls()
{
	if (isset($_POST['url']))
	{
		$urls = unserialize(get_option('wpex_replace_exclude_search_urls'));
		if (empty($urls))
			$urls = array();

		if (!in_array($_POST['url'], $urls))
			$urls[] = $_POST['url'];
			
		update_option('wpex_replace_exclude_search_urls', serialize($urls));
		die(json_encode(array(
			'error' => false,
		)));		
	}
	
	die(json_encode(array(
		'error' => true,
		'message' => __('Invalid function call', 'wpex-replace'),
	)));
}

// creating Ajax call for include url to search
add_action( 'wp_ajax_wpex_replace_include_search_url', 'wpex_replace_ajax_include_search_url' );
function wpex_replace_ajax_include_search_url()
{
	if (isset($_POST['url']))
	{
		$urls = unserialize(get_option('wpex_replace_exclude_search_urls'));
		if (empty($urls))
			$urls = array();

		if (($key = array_search($_POST['url'], $urls)) !== false)
		{
			unset($urls[$key]);
		}

		update_option('wpex_replace_exclude_search_urls', serialize($urls));
		die(json_encode(array(
			'error' => false,
		)));		
	}

	die(json_encode(array(
		'error' => true,
		'message' => __('Invalid function call', 'wpex-replace'),
	)));
}

// creating Ajax call for replacing the strings
add_action( 'wp_ajax_wpex_replace_replace_strings', 'wpex_replace_ajax_replace_strings' );
function wpex_replace_ajax_replace_strings()
{
	global $wpdb;
	
	if (isset($_POST['data']))
	{
		$r = new WPEXReplaceTextInTables();

		$error = $r->replace($_POST['data']);
		if ($error)
			$html = '<div class="error">'.$error.'<div>';
		else
			$html = '<p>'.__('Urls have been replaced', 'wpex-replace').'</p>';
		
		die(json_encode(array(
			'html' => $html,
		)));
	}

	die(json_encode(array(
		'error' => true,
		'message' => __('Invalid function call', 'wpex-replace'),
	)));
}