<?php

class WPEXReplaceTextInTables
{
	private $results = '';

	function getExcludedInternalUrls()
	{
		static $urls = null;
		if (!$urls)
			$urls = include(__DIR__.'/excluded-urls.php');
		return $urls;
	}

	function getExcludedUserDefinedUrls()
	{
		$urls = unserialize(get_option('wpex_replace_exclude_search_urls'));
		if (empty($urls))
			$urls = array();
		return $urls;
	}
	
	function getExcludedUrlList()
	{
		return array_merge($this->getExcludedUserDefinedUrls(), $this->getExcludedInternalUrls());
	}

	function _addUrlToList(&$result, $table, $url, $idName, $idValue)
	{
		if (!array_key_exists($url, $result))
			$result[$url] = array();

		if (!array_key_exists($table, $result[$url]))
			$result[$url][$table] = array();

		foreach ($result[$url][$table] as $item)
		{
			if ($item['name']==$idName && $item['value']==$idValue)
				return;
		}
		
		$result[$url][$table][] = array('name'=>$idName, 'value'=>$idValue);
	}
	
	function findURls()
	{
		$replaceList = array(
			'http://'=>'http://',
			'https://'=>'https://',
		);
		
		$result = array();
		$regex = "!((?:".implode('|', array_keys($replaceList)).")[^#?/\"]*)(\/wp-content)?!";
		
		$tables = $this->getTables();
		foreach ($tables as $table)
		{
			$columns = $this->getColumns($table);
			
			$rows = $this->getData($table, $replaceList);
			foreach ($rows as $row)
			{
				foreach ($columns as $column)
				{
					$matchResult = preg_match_all($regex, $row->$column, $matches, PREG_SET_ORDER);
					if ($matchResult > 0)
					{
						foreach ($matches as $match)
						{
							$url = trim($match[1], "/");
							if (strpos($url, ' '))
								continue;
							if (in_array($url, $this->getExcludedUrlList()))
								continue;
							
							$this->_addUrlToList($result, $table, $url, $columns[0], $row->$columns[0]);
						}
					}
				}
			}
		}
		return $result;
	}
	
	
	function replace($data)
	{
    global $wpdb;
    
		$error = '';
		
		foreach($_POST['data'] as $item)
		{
			$replaceList = array($item['oldText'] => $item['newText']);

			preg_match_all('/([^\[]+)\[([^\]]+)\]=([^;]+);/', $item['ids'], $matches, PREG_SET_ORDER);
			foreach ($matches as $match)
			{
				// set array as variables
				list($dummy, $table, $idName, $idValues) = $match;

				// table columns
				$columns = $this->getColumns($table);

				// selected fields
				$fields = "";
				foreach ($columns as $column)
				{
					if ($fields)
						$fields .= ",";
					$fields .= "`{$column}`";
				}
		
				// get the rows that should be replaced
				$query = "select {$fields} from `{$table}` where `{$idName}` in ({$idValues})";
				$dbResult = $wpdb->get_results($query, OBJECT);

				// find columns that has changed
				foreach ($dbResult as $row)
				{
					$modifiedRow = new StdClass();
					$modifiedRow->$idName = $row->$idName;
					
					// loop over all columns
					foreach ($row as $key=>$value)
					{
						// ignore id
						if ($key == $idName)
							continue;

						// replace data
						$row->$key = $this->replaceValue($value, $replaceList);

						// remember results
						if (strcasecmp($value, $row->$key) != 0)
						{
							$modifiedRow->$key = $row->$key;
						}
					}

					// save changes
					if (!$this->saveData($table, $idName, $modifiedRow))
					{
						$error .= str_replace(
							array('{table}', '{id_name}', '{id}'),
							array($table, $idName, $id),
							__('table {table} {id_name}={id} could not be saved.', 'wpex-replace').'<br/>'
						);
					}
				}
			}
		}
		
		return $error;
	}
	
	function replaceValue($value, $replaceList)
	{
		if (is_array($value))
		{
			$a = array();
			foreach ($value as $k=>$v)
			{
				$a[$k] = $this->replaceValue($v, $replaceList);
			}
			return $a;
		}
		if (strncmp($value, 'a:', 2) == 0)
		{
			$a = unserialize($value);
			foreach ($a as $k=>$v)
			{
				$a[$k] = $this->replaceValue($v, $replaceList);
			}
			return serialize($a);
		}
	
		$oldValue = $value;
		$newValue = str_replace(array_keys($replaceList), array_values($replaceList), $value);
		
		if (strcasecmp($oldValue, $newValue) == 0)
			return $value;
		
		//// remember results
		//$displayOld = htmlspecialchars($oldValue);
		//$displayNew = htmlspecialchars($newValue);
		//foreach ($replaceList as $k=>$v)
		//{
		//	$displayOld = str_replace($k, '<span class="marked">'.$k.'</span>', $displayOld);
		//	$displayNew = str_replace($v, '<span class="marked">'.$v.'</span>', $displayNew);
		//}
		//$this->result .= '<div class="row value"><div class="old-site">'.$displayOld.'</div><div class="new-site">'.$displayNew.'</div><div class="clearfix"></div></div>';
		
		return $newValue;
	}
	
	function getTables()
	{
		global $wpdb;
		
		$query = "SHOW TABLES FROM `".DB_NAME."`";
    $dbResult = $wpdb->get_results($query, OBJECT);

		$tables = array();
		if ($dbResult)
    {
			$fieldName = "Tables_in_".DB_NAME;
      foreach ($dbResult as $table)
      {
				$tables[] = $table->$fieldName;
      };
		}
		return $tables;
	}
	
	function getColumns($tableName)
	{
		global $wpdb;
		
		$query = "SHOW COLUMNS FROM `{$tableName}` WHERE `type` LIKE '%char%' OR `type` LIKE '%text%' OR Extra = 'auto_increment'";
    $dbResult = $wpdb->get_results($query, OBJECT);

		$columns = array();
		if ($dbResult)
    {
      foreach ($dbResult as $column)
      {
				$columns[] = $column->Field;
      };
		}
		return $columns;
	}
	
	function getData($tableName, $replaceList)
	{
		global $wpdb;
		
		$rows = array();

		$columns = $this->getColumns($tableName);
		if (empty($columns) || count($columns)<=0)
			return $rows;

		// store the id to ignore it
		$id = $columns[0];

		// build where clause
		$where = "";
		foreach ($columns as $column)
		{
			// ignore id
			if ($column == $id)
				continue;
			
			foreach ($replaceList as $key=>$value)
			{
				if ($where)
					$where .= " OR ";
				$where .= "`{$column}` like '%{$key}%'";
			}
		}

		// get data
		$query = "SELECT `".implode("`,`", $columns)."` FROM `{$tableName}` WHERE {$where}";
    $dbResult = $wpdb->get_results($query, OBJECT);
			
		// build response array 
		if ($dbResult)
    {
      foreach ($dbResult as $row)
      {
				$rows[] = $row;
      };
		}
		return $rows;
	}
	
	function saveData($tableName, $idName, $row)
	{
		// build set of query
		$set = '';
		foreach ($row as $key=>$value)
		{
			// ignore id
			if ($key == $idName)
				continue;
			
			if ($set)
				$set .= ",";
			$set .= "`{$key}`='".mysql_escape_string($value)."'";
		}
		if (empty($set))
			return true;
		
		$query = "UPDATE `{$tableName}` SET {$set} WHERE {$idName}={$row->$idName}";
		return (mysql_query($query) === true);
	}
}
